/*---------------------------------------------------------------------------
LISTING 1.   32-POINT DIRECT WALSH TRANSFORM IN C.

32-point fast Walsh transform by Carey Witkov,  Dept. of Physics,
Broward Community College, 1990 May 01.  Coded for JFORTH, a product
of Delta Research, Inc.

Revisions:
1990 Oct 10	Translated from Forth to C by Darwin Engwer (CIS 71121,705).
---------------------------------------------------------------------------*/

#include <stdio.h>


#define RESULTS_PER_LINE	8	/* Output format control constant. */
#define N			32	/* N = 32 for sample data. */
#define K			5	/* log2 (N) */

long GrayArray [N];	/* Array of gray code values. */

/*
Rademacher functions for 32-point data; binary 0 represents 1;
binary 1 represents -1.
*/
long RadArray [K + 1] =
	{
	0L,
	0x0000ffffL,
	0x00ff00ffL,
	0x0f0f0f0fL,
	0x33333333L,
	0x55555555L
	};

long WalshArray [N];	/* Array of Walsh functions, in sequency order. */

/* Data array. */
long x [N] =
	{
        /* Sample data; 32-point sampled sine wave; scaling factor = 1000 */
         195L,  383L,  556L,  707L,  831L,  924L,  981L,  1000L,
	 981L,  924L,  831L,  707L,  556L,  383L,  195L,     0L,
	-195L, -383L, -556L, -707L, -831L, -924L, -981L, -1000L,
	-981L, -924L, -831L, -707L, -556L, -383L, -195L,     0L
	};

long XT [N];	/* Walsh-transformed output array. */



/*---------------------------------------------------------------------------
Converts a binary value to the equivalent Gray code value.
---------------------------------------------------------------------------*/
static long GrayEncoder (long Binary)

	{
	return (Binary ^ (Binary / 2));
	}



/*---------------------------------------------------------------------------
Fills a 32-cell Gray code look-up-table called GrayArray.
---------------------------------------------------------------------------*/
static void FillGrayArray (void)

	{
	int i;

	for (i = 0;  i < N;  i++)
		GrayArray [i] = GrayEncoder ((long) i);
	}



/*---------------------------------------------------------------------------
Returns the state a bit (true or false) in a given data value.
---------------------------------------------------------------------------*/
int BitTest (long Data, int BitNumber)

	{
	long Mask = 1L;

	return ((Data & (Mask << BitNumber)) != 0L);
	}



/*---------------------------------------------------------------------------
Fills a 32-cell Walsh function look-up-table called WalshArray.
---------------------------------------------------------------------------*/
static void FillWalshArray (void)

	{
	int i, i1;
	long WalshValue;

	for (i = 0;  i < N;  i++)
		{
		WalshValue = 0L;
		for (i1 = 0;  i1 < K;  i1++)  /* 1990/10/10 DAE  K not K+1 ! */
			if (BitTest (GrayArray [i], i1))
				WalshValue ^= RadArray [i1 + 1];

		WalshArray [i] = WalshValue;
		}
	}



/*---------------------------------------------------------------------------
Unnormalized direct Walsh transform of 32-point data contained in x array.
Transformed output is in XT array.
Requires FillGrayArray and FillWalshArray.
---------------------------------------------------------------------------*/
static void WalshTransform (void)

	{
	int i, i1;
	long TransformValue;
	int Counter = 0;

	for (i = 0;  i < N;  i++)
		{
		TransformValue = 0L;
		for (i1 = 0;  i1 < N;  i1++)
			if (BitTest (WalshArray [Counter], i1))
				TransformValue -= x [N - 1 - i1];
			else
				TransformValue += x [N - 1 - i1];

		XT [i] = TransformValue;
		Counter++;
		}
	}



/*---------------------------------------------------------------------------
Single routine that performs unnormalized direct walsh transform of
32-point data contained in x array, leaving transformed output in XT array.
---------------------------------------------------------------------------*/
static void DirectWalshTransform (void)

	{
	FillGrayArray ();
	FillWalshArray ();
	WalshTransform ();
	}



/*---------------------------------------------------------------------------
Normalizes and prints the transformed output array.
---------------------------------------------------------------------------*/
static void Check (void)

	{
	int i;

	for (i = 0;  i < N;  i++)
        	printf ("%5ld%c", XT [i] / N,
			((i + 1) % RESULTS_PER_LINE) ? ' ' : '\n');
	}



int main (void)

	{
	DirectWalshTransform ();
	Check ();

	return (0);
	}


/* The End. */
